
#include "stdafx.h"
#include "Tinter.h"
#include "GifPreviewBar.h"

#define SWATCH_START 1000

//====================
// CGifPreviewBar

IMPLEMENT_DYNAMIC(CGifPreviewBar, CDialogBar)

CGifPreviewBar::CGifPreviewBar()
{
}

CGifPreviewBar::~CGifPreviewBar()
{
}

BEGIN_MESSAGE_MAP(CGifPreviewBar, CDialogBar)
	ON_MESSAGE(WM_INITDIALOG, OnInitDialog)
	ON_UPDATE_COMMAND_UI(IDC_PALETTE, OnUpdatePalette)
END_MESSAGE_MAP()

LRESULT CGifPreviewBar::OnInitDialog(WPARAM /*wParam*/, LPARAM /*lParam*/)
{
	c_PaletteCtrl.SubclassDlgItem(IDC_PALETTE, this);
	return Default();
}

void CGifPreviewBar::SetPalette(Image * pImage)
{
	UINT nColors = c_PaletteCtrl.SetPalette(pImage);
	SetDlgItemInt(IDC_COLORCOUNT, nColors, FALSE);
}

void CGifPreviewBar::OnUpdatePalette(CCmdUI * pCmdUI)
{
	// needed to enable palette tooltips
}

//====================
// CColorPaletteCtrl

// VC++ 7.0 supports owner draw CStatic, but VC++ 6.0 doesn't.
// That's why we derive CColorPaletteCtrl from CButton,
// although it really is more like a CStatic.

IMPLEMENT_DYNAMIC(CColorPaletteCtrl, CButton)

CColorPaletteCtrl::CColorPaletteCtrl()
: m_pPalette(NULL)
{
}

CColorPaletteCtrl::~CColorPaletteCtrl()
{
	delete[] m_pPalette;
}

BEGIN_MESSAGE_MAP(CColorPaletteCtrl, CButton)
	ON_NOTIFY(TTN_NEEDTEXT, 0, OnToolTipNeedText)
END_MESSAGE_MAP()

UINT CColorPaletteCtrl::SetPalette(Image * pImage)
{
	delete[] m_pPalette;
	m_pPalette = NULL;

	UINT r(0);

	if (pImage)
	{
		int nBytes = pImage->GetPaletteSize();
		m_pPalette = new BYTE[nBytes];
		if (m_pPalette)
		{
			ColorPalette * pPal = (ColorPalette *) m_pPalette;
			if (pImage->GetPalette(pPal, nBytes) == Ok) r = pPal->Count;
			else
			{
				delete[] m_pPalette;
				m_pPalette = NULL;
			}
		}
	}

	Invalidate();
	return r;
}

void CColorPaletteCtrl::DrawItem(LPDRAWITEMSTRUCT lpDS)
{
	if (! m_pPalette) return;
	ColorPalette * pPal = (ColorPalette *) m_pPalette;

	CRect rc(lpDS->rcItem);
	CDC dc;
	dc.Attach(lpDS->hDC);

	dc.FillSolidRect(rc, ::GetSysColor(COLOR_BTNFACE));

	UINT dx = rc.Width() / 16;
	UINT dy = rc.Height() / 16;

	UINT xMargin = (rc.Width() - 16 * dx) / 2;
	UINT yMargin = (rc.Height() - 16 * dy) / 2;

	UINT w = dx - ::GetSystemMetrics(SM_CXBORDER);
	UINT h = dy - ::GetSystemMetrics(SM_CYBORDER);

	UINT iCol = 0;

	UINT x = xMargin;
	UINT y = yMargin;

	bool bAddTools = (m_ToolTips.GetToolCount() == 0);

	for (UINT row = 0; row < 16; row++)
	{
		for (UINT col = 0; col < 16; col++)
		{
			CRect rcSwatch(x, y, x + w, y + h);

			if (iCol < pPal->Count)
			{
				Color color(pPal->Entries[iCol]);
				dc.FillSolidRect(rcSwatch, color.ToCOLORREF());
			}

			if (bAddTools)
				m_ToolTips.AddTool(this, LPSTR_TEXTCALLBACK, rcSwatch, SWATCH_START + iCol);

			++iCol;
			x += dx;
		}
		x = xMargin;
		y += dy;
	}

	dc.Detach();
}

void CColorPaletteCtrl::OnToolTipNeedText(NMHDR * pNMH, LRESULT * /*result*/)
{
	ASSERT(m_pPalette);
	ColorPalette * pPal = (ColorPalette *) m_pPalette;

	LPNMTTDISPINFO pInfo = (LPNMTTDISPINFO) pNMH;
	UINT id = (UINT) pInfo->hdr.idFrom - SWATCH_START;

	if (id < pPal->Count)
	{
		Color color(pPal->Entries[id]);

		CString s;
		s.Format(_T("R%d  G%d  B%d"), color.GetR(), color.GetG(), color.GetB());
		_tcscpy(pInfo->szText, s);
	}
}

BOOL CColorPaletteCtrl::PreTranslateMessage(MSG* pMsg)
{
	m_ToolTips.RelayEvent(pMsg);
	return CButton::PreTranslateMessage(pMsg);
}

void CColorPaletteCtrl::PreSubclassWindow()
{
	m_ToolTips.Create(this);
}
