// MainFrm.cpp
//
// Part of Tinter, a small app to demonstrate QColorMatrix and QGaussFilter.
// Use at your own risk. Comments welcome.
//
// (c) 2003, Sjaak Priester, Amsterdam.
// mailto:sjaak@sjaakpriester.nl

#include "stdafx.h"
#include "Resource.h"

#include "MainFrm.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif

const int TimerID = 3;
const int TimerPeriod = 4000;
const int FirstMessage = 1000;

IMPLEMENT_DYNCREATE(CMainFrame, CFrameWnd)

BEGIN_MESSAGE_MAP(CMainFrame, CFrameWnd)
	ON_WM_CREATE()
	ON_WM_DESTROY()
	ON_WM_TIMER()
END_MESSAGE_MAP()

static UINT indicators[] =
{
	ID_SEPARATOR,
	ID_INDICATOR_CALCULATING,
	ID_INDICATOR_GIF,
	ID_INDICATOR_MODIFIED,
	ID_INDICATOR_SIZE,
	ID_INDICATOR_SCALE
};

CMainFrame::CMainFrame()
: m_iMessage(FirstMessage)
{
}

CMainFrame::~CMainFrame()
{
}

int CMainFrame::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
	if (CFrameWnd::OnCreate(lpCreateStruct) == -1)
		return -1;
	
	if (!m_wndToolBar.CreateEx(this, TBSTYLE_FLAT, WS_CHILD | WS_VISIBLE | CBRS_TOP
		| CBRS_GRIPPER | CBRS_TOOLTIPS | CBRS_FLYBY | CBRS_SIZE_DYNAMIC) ||
		!m_wndToolBar.LoadToolBar(IDR_MAINFRAME))
	{
		TRACE0("Failed to create toolbar\n");
		return -1;
	}

	if (!m_wndStatusBar.Create(this) ||
		!m_wndStatusBar.SetIndicators(indicators,
		  sizeof(indicators)/sizeof(UINT)))
	{
		TRACE0("Failed to create status bar\n");
		return -1;
	}

	if (! m_SliderBar.Create(this, IDD_SLIDERBAR,
		CBRS_RIGHT,
		IDD_SLIDERBAR))
	{
		TRACE0("Failed to create slider bar\n");
		return -1;
	}

	if (! m_GifPreviewBar.Create(this, IDD_GIFPREVIEWBAR,
		CBRS_RIGHT,
		IDD_GIFPREVIEWBAR))
	{
		TRACE0("Failed to create GIF preview bar\n");
		return -1;
	}
	ShowControlBar(& m_GifPreviewBar, FALSE, FALSE);

	EnableDocking(CBRS_ALIGN_ANY);
	m_SliderBar.EnableDocking(CBRS_ALIGN_RIGHT);
	m_GifPreviewBar.EnableDocking(CBRS_ALIGN_RIGHT);
	m_wndToolBar.EnableDocking(CBRS_ALIGN_ANY);
	DockControlBar(&m_wndToolBar);

	SetTimer(TimerID, TimerPeriod, NULL);
	return 0;
}

void CMainFrame::OnDestroy()
{
	CFrameWnd::OnDestroy();
	KillTimer(TimerPeriod);
}

void CMainFrame::OnTimer(UINT nIDEvent)
{
	if (nIDEvent == TimerID)
	{
		CString s;
		if (! s.LoadString(m_iMessage++))
		{
			m_iMessage = FirstMessage;
			s.LoadString(m_iMessage++);
		}
		SetMessageText(s);
	}
	CFrameWnd::OnTimer(nIDEvent);
}
