// SliderBar.cpp
//
// Part of Tinter, a small app to demonstrate QColorMatrix.
// Use at your own risk. Comments welcome.
//
// Version 1.1
// Added: support for Unsharp Mask/Blur
//
// (c) 2003, Sjaak Priester, Amsterdam.
// mailto:sjaak@sjaakpriester.nl

#include "stdafx.h"
#include "resource.h"
#include "SliderBar.h"

#include <math.h>

// The information of the sliders is kept in a table.
struct SliderInfo_
{
	UINT id;		// dialog control ID of the slider
	UINT idBuddy;	// dialog control ID of the static buddy window
	int rangeMin;	// minimum position of the slider
	int rangeMax;	// maximum position
	int neutral;	// neutral position
	REAL factor;	// external value = factor * slider position
	int decimals;	// number of decimals for display in buddy
};

// The slider information table.
SliderInfo_ SliderInfo[] =
{
	{ IDC_CONTRAST, IDCS_CONTRAST, 4, 80, 20, 0.05f, 2 },
	{ IDC_BRIGHTNESS, IDCS_BRIGHTNESS, -20, 20, 0, 0.05f, 2 },
	{ IDC_SATURATION, IDCS_SATURATION, 0, 60, 20, 0.05f, 2 },
	{ IDC_HUE, IDCS_HUE, -45, 45, 0, 4.0f, 0 },
	{ IDC_GAMMA, IDCS_GAMMA, 4, 80, 20, 0.05f, 2 },
	{ IDC_TINTHUE, IDCS_TINTHUE, -45, 45, 0, 4.0f, 0 },
	{ IDC_TINTAMOUNT, IDCS_TINTAMOUNT, -20, 20, 0, 0.05f, 2 },
	{ IDC_RADIUS, IDCS_RADIUS, 0, 80, 7, 0.1f, 2 },
	{ IDC_DEPTH, IDCS_DEPTH, 0, 40, 8, 0.1f, 2 },
};

// The number of sliders.
const int nSliders = sizeof(SliderInfo)/sizeof(SliderInfo[0]);

IMPLEMENT_DYNAMIC(CSliderBar, CDialogBar)

CSliderBar::CSliderBar()
{
}

CSliderBar::~CSliderBar()
{
}

BEGIN_MESSAGE_MAP(CSliderBar, CDialogBar)
	ON_MESSAGE(WM_INITDIALOG, OnInitDialog)
	ON_WM_HSCROLL()
	ON_BN_CLICKED(IDC_NEUTRAL, OnBnClickedNeutral)
	ON_UPDATE_COMMAND_UI(IDC_NEUTRAL, OnUpdateCommandNeutral)
	ON_CBN_SELENDOK(IDC_FILTER, OnCbnSelendOkFilter)
	ON_BN_CLICKED(IDC_LUMINANCE, OnBnClickedLuminance)
	ON_BN_CLICKED(IDC_NEGATIVE, OnBnClickedNegative)
END_MESSAGE_MAP()

LRESULT CSliderBar::OnInitDialog(WPARAM /*wParam*/, LPARAM /*lParam*/)
{
	SliderInfo_ * pInfo = SliderInfo;

	for (int i = 0; i < nSliders; i++)
	{
		CSliderCtrl * pSlider = (CSliderCtrl *) GetDlgItem(pInfo->id);
		pSlider->SetRange(pInfo->rangeMin, pInfo->rangeMax);
		pSlider->SetBuddy(GetDlgItem(pInfo->idBuddy), FALSE);
		pInfo++;
	}

	SetNeutral();
	GetDlgItem(IDC_FILTER)->PostMessage(CB_SETCURSEL, 0);
	return Default();
}

void CSliderBar::OnHScroll(UINT nSBCode, UINT /*nPos*/, CScrollBar* pScrollBar)
{
	CSliderCtrl * pSlider = (CSliderCtrl *) pScrollBar;
	UpdateBuddy(pSlider);

	// TB_ENDTRACK and TB_THUMBPOSITION always come after another message
	// about the same position, so they don't carry useful information for us.
	if (nSBCode == TB_ENDTRACK || nSBCode == TB_THUMBPOSITION) return;

	NotifyView(pSlider->GetDlgCtrlID());
}

void CSliderBar::OnBnClickedNeutral()
{
	SetNeutral();
	NotifyView(0);
}

void CSliderBar::OnUpdateCommandNeutral(CCmdUI * pCmdUI)
{
	pCmdUI->Enable(IsDirty());
}

bool CSliderBar::IsDirty(void)
{
	bool bDirty = false;

	SliderInfo_ * pInfo = SliderInfo;

	for (int i = 0; i < nSliders; i++)
	{
		CSliderCtrl * pSlider = (CSliderCtrl *) GetDlgItem(pInfo->id);
		if (pSlider->GetPos() != pInfo->neutral)
		{
			bDirty = true;
			break;
		}
		pInfo++;
	}
	CComboBox * pFilter = (CComboBox *) GetDlgItem(IDC_FILTER);
	if (pFilter->GetCurSel() != 0) bDirty = true;

	return bDirty;
}

int CSliderBar::GetFilterMode() const
{
	CComboBox * pFilter = (CComboBox *) GetDlgItem(IDC_FILTER);
	return pFilter->GetCurSel();
}

REAL CSliderBar::GetValue(UINT id) const
{
	SliderInfo_ * pInfo = SliderInfo;

	for (int i = 0; i < nSliders; i++)
	{
		if (id == pInfo->id)
		{
			CSliderCtrl * pSlider = (CSliderCtrl *) GetDlgItem(pInfo->id);
			return (REAL) pSlider->GetPos() * pInfo->factor;
		}
		pInfo++;
	}
	return 0.0f;
}

void CSliderBar::SetValue(UINT id, REAL value)
{
	SliderInfo_ * pInfo = SliderInfo;

	for (int i = 0; i < nSliders; i++)
	{
		if (id == pInfo->id)
		{
			CSliderCtrl * pSlider = (CSliderCtrl *) GetDlgItem(pInfo->id);
			pSlider->SetPos((int) floor(value / pInfo->factor + 0.5f));
			UpdateBuddy(pSlider);
		}
		pInfo++;
	}
}

void CSliderBar::SetNeutral(void)
{
	CButton * pNegative = (CButton *) GetDlgItem(IDC_NEGATIVE);
	pNegative->SetCheck(0);
	CComboBox * pFilter = (CComboBox *) GetDlgItem(IDC_FILTER);
	pFilter->SetCurSel(0);
	CButton * pLuminance = (CButton *) GetDlgItem(IDC_LUMINANCE);
	pLuminance->SetCheck(1);

	EnableFilterSliders();

	SliderInfo_ * pInfo = SliderInfo;

	for (int i = 0; i < nSliders; i++)
	{
		CSliderCtrl * pSlider = (CSliderCtrl *) GetDlgItem(pInfo->id);
		pSlider->SetPos(pInfo->neutral);
		UpdateBuddy(pSlider);
		pInfo++;
	}
}

void CSliderBar::UpdateBuddy(CSliderCtrl * pSlider)
{
	UINT id = pSlider->GetDlgCtrlID();

	CWnd * pBuddy = pSlider->GetBuddy(FALSE);
	if (pBuddy)
	{
		SliderInfo_ * pInfo = SliderInfo;

		for (int i = 0; i < nSliders; i++)
		{
			if (id == pInfo->id)
			{
				CString s;
				s.Format(_T("%0.*f"), pInfo->decimals, GetValue(id));
				pBuddy->SetWindowText(s);
				return;
			}
			pInfo++;
		}
	}
}

LRESULT CSliderBar::NotifyView(UINT id)
{
	// Make it work in MFC SDI and MDI apps, with or without doc-view
	CFrameWnd * pFrame = (CFrameWnd*) ::AfxGetMainWnd();
	pFrame = pFrame->GetActiveFrame();

	CWnd * pView = pFrame->GetActiveView();

	if (! pView) pView = pFrame->GetDlgItem(AFX_IDW_PANE_FIRST);
	ASSERT(pView);

	LRESULT r = 0;
	if (pView) r = pView->SendMessage(QM_SLIDERBAR, (WPARAM) id);
	return r;
}

void CSliderBar::OnCbnSelendOkFilter()
{
	EnableFilterSliders();
	NotifyView(IDC_FILTER);
}

void CSliderBar::EnableFilterSliders(void)
{
	CComboBox * pFilter = (CComboBox *) GetDlgItem(IDC_FILTER);
	int sel = pFilter->GetCurSel();
	if (sel < 0) return;

	GetDlgItem(IDC_RADIUS)->EnableWindow(sel > 0);
	GetDlgItem(IDCS_RADIUS)->EnableWindow(sel > 0);
	GetDlgItem(IDC_DEPTH)->EnableWindow(sel == 1);
	GetDlgItem(IDCS_DEPTH)->EnableWindow(sel == 1);
	GetDlgItem(IDC_LUMINANCE)->EnableWindow(sel > 0);
}

void CSliderBar::OnBnClickedLuminance()
{
	NotifyView(IDC_LUMINANCE);
}

void CSliderBar::OnBnClickedNegative()
{
	NotifyView(IDC_NEGATIVE);
}
