// CSliderBar.h
//
// Part of Tinter, a small app to demonstrate QColorMatrix.
// Use at your own risk. Comments welcome.
//
// Version 1.1
// (c) 2003, Sjaak Priester, Amsterdam.
// mailto:sjaak@sjaakpriester.nl

#pragma once

// Special Windows message to notify the View of any
// change in the slider positions.
// WPARAM is one of IDC_CONTRAST... IDC_LUMINANCE, or zero for Set Neutral.
// LPARAM is not used.
#define QM_SLIDERBAR	WM_APP

class CSliderBar : public CDialogBar
{
public:
	CSliderBar();
	virtual ~CSliderBar();

	enum { IDD = IDD_SLIDERBAR };

	enum
	{
		FilterOff = 0,
		FilterUnsharpMask = 1,
		FilterBlur = 2
	};
	int GetFilterMode() const;
	bool GetLuminance() const
			{ return ((CButton *) GetDlgItem(IDC_LUMINANCE))->GetCheck() != 0; }
	bool GetNegative() const
			{ return ((CButton *) GetDlgItem(IDC_NEGATIVE))->GetCheck() != 0; }

	// Get the values of the sliders in 'external' format.
	REAL GetContrast() const			{ return GetValue(IDC_CONTRAST); }
	REAL GetBrightness() const			{ return GetValue(IDC_BRIGHTNESS); }
	REAL GetSaturation() const			{ return GetValue(IDC_SATURATION); }
	REAL GetHue() const					{ return GetValue(IDC_HUE); }
	REAL GetGamma() const				{ return GetValue(IDC_GAMMA); }
	REAL GetTintHue() const				{ return GetValue(IDC_TINTHUE); }
	REAL GetTintAmount() const			{ return GetValue(IDC_TINTAMOUNT); }
	REAL GetRadius() const				{ return GetValue(IDC_RADIUS); }
	REAL GetDepth() const				{ return GetValue(IDC_DEPTH); }

	// Set the values.
	void SetContrast(REAL contrast)		{ SetValue(IDC_CONTRAST, contrast); }
	void SetBrightness(REAL brightness)	{ SetValue(IDC_BRIGHTNESS, brightness); }
	void SetSaturation(REAL saturation)	{ SetValue(IDC_SATURATION, saturation); }
	void SetHue(REAL hue)				{ SetValue(IDC_HUE, hue); }
	void SetGamma(REAL gamma)			{ SetValue(IDC_GAMMA, gamma); }
	void SetTintHue(REAL hue)			{ SetValue(IDC_TINTHUE, hue); }
	void SetTintAmount(REAL amount)		{ SetValue(IDC_TINTAMOUNT, amount); }
	void SetRadius(REAL radius)			{ SetValue(IDC_RADIUS, radius); }
	void SetDepth(REAL depth)			{ SetValue(IDC_DEPTH, depth); }

	// Set the sliders in the neutral position.
	void SetNeutral(void);

	// true if at least one of the sliders is not in the neutral position.
	bool IsDirty(void);

protected:
	afx_msg LRESULT OnInitDialog(WPARAM wParam, LPARAM lParam);
	afx_msg void OnHScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar);
	afx_msg void OnBnClickedNeutral();
	afx_msg void OnUpdateCommandNeutral(CCmdUI * pCmdUI);
	afx_msg void OnCbnSelendOkFilter();
	afx_msg void OnBnClickedLuminance();
	afx_msg void OnBnClickedNegative();

	REAL GetValue(UINT id) const;
	void SetValue(UINT id, REAL value);
	void UpdateBuddy(CSliderCtrl * pSlider);
	void EnableFilterSliders(void);
	LRESULT NotifyView(UINT id);

	DECLARE_DYNAMIC(CSliderBar)
	DECLARE_MESSAGE_MAP()
};
