// TinterView.h : interface of the CTinterView class
//
// Part of Tinter, a small app to demonstrate QColorMatrix and QConvoluter.
// Use at your own risk. Comments welcome.
//
// Version 1.3
// (c) 2003-2004, Sjaak Priester, Amsterdam.
// mailto:sjaak@sjaakpriester.nl

#pragma once
#include "QGaussFilter.h"
#include "QColorMatrix.h"
#include "QZoomView.h"
#include "TwainCPP.h"

class CTinterView : public QZoomView, public CTwain
{
protected: // create from serialization only
	CTinterView();
	virtual ~CTinterView();

	virtual BOOL OnPreparePrinting(CPrintInfo* pInfo);
	virtual void OnBeginPrinting(CDC* pDC, CPrintInfo* pInfo);
	virtual void OnEndPrinting(CDC* pDC, CPrintInfo* pInfo);

	virtual BOOL PreCreateWindow(CREATESTRUCT& cs);
	virtual BOOL PreTranslateMessage(MSG* pMsg);
	virtual void OnPrepareDC(CDC* pDC, CPrintInfo* pInfo = NULL);
	virtual void OnDraw(CDC* pDC);
	virtual void OnPrint(CDC* pDC, CPrintInfo* pInfo);
	virtual void OnInitialUpdate();
	virtual void OnUpdate(CView* pSender, LPARAM lHint, CObject* pHint);

	// overrides CTwain
	virtual void GetIdentity(void);
	virtual void CopyImage(HANDLE hBitmap, TW_IMAGEINFO& info);

	afx_msg void OnSetNeutral();
	afx_msg void OnUpdateGeneral(CCmdUI *pCmdUI);
	afx_msg void OnFileSaveoptions();
	afx_msg void OnViewModified();
	afx_msg void OnUpdateViewModified(CCmdUI *pCmdUI);
	afx_msg void OnUpdateModified(CCmdUI *pCmdUI);
	afx_msg void OnUpdateCalculating(CCmdUI *pCmdUI);
	afx_msg void OnUpdateScale(CCmdUI *pCmdUI);
	afx_msg void OnUpdateSize(CCmdUI *pCmdUI);
	afx_msg void OnViewBorder();
	afx_msg void OnUpdateViewBorder(CCmdUI *pCmdUI);
	afx_msg void OnFileUseimageresolution();
	afx_msg void OnUpdateFileUseimageresolution(CCmdUI *pCmdUI);
	afx_msg void OnViewZoomin();
	afx_msg void OnUpdateViewZoomin(CCmdUI *pCmdUI);
	afx_msg void OnViewZoomout();
	afx_msg void OnUpdateViewZoomout(CCmdUI *pCmdUI);
	afx_msg void OnViewZoomtowindow();
	afx_msg void OnViewActualsize();
	afx_msg void OnToolPlusloupe();
	afx_msg void OnUpdateToolPlusloupe(CCmdUI *pCmdUI);
	afx_msg void OnToolMinusloupe();
	afx_msg void OnUpdateToolMinusloupe(CCmdUI *pCmdUI);
	afx_msg void OnToolHand();
	afx_msg void OnUpdateToolHand(CCmdUI *pCmdUI);
	afx_msg void OnZoom(UINT nID);
	afx_msg void OnUpdateZoom(CCmdUI *pCmdUI);
	afx_msg void OnZoomOther();
	afx_msg void OnViewProperties();
	afx_msg void OnViewBackgroundcolor();
	afx_msg void OnViewBordercolor();
	afx_msg void OnFileSelectsource();
	afx_msg void OnUpdateFileSelectsource(CCmdUI *pCmdUI);
	afx_msg void OnFileAcquire();
	afx_msg void OnUpdateFileAcquire(CCmdUI *pCmdUI);
	afx_msg void OnFlipHorizontal();
	afx_msg void OnFlipVertical();
	afx_msg void OnRotate180();
	afx_msg void OnRotate270();
	afx_msg void OnRotate90();
	afx_msg void OnUpdateRotateFlip(CCmdUI * pCmdUI);
	afx_msg int OnCreate(LPCREATESTRUCT lpCreateStruct);
	afx_msg BOOL OnEraseBkgnd(CDC* pDC);
	afx_msg LRESULT OnSliderBar(WPARAM wParam, LPARAM lParam);
	afx_msg LRESULT OnGaussFilter(WPARAM wParam, LPARAM lParam);
	afx_msg void OnViewGifpreview();
	afx_msg void OnUpdateViewGifpreview(CCmdUI *pCmdUI);
	afx_msg void OnUpdateGifpreview(CCmdUI *pCmdUI);
	afx_msg void OnCloseGifPreview();
	afx_msg void OnGifMode(UINT nID);
	afx_msg void OnUpdateGifMode(CCmdUI *pCmdUI);
	afx_msg void OnEditSize();

	Status Save(LPCTSTR pPath);
	void SetBitmap(Bitmap * pSrc);
	void SetSizes(void);
	void FilterOff(void);
	void FitInWindow(void);
	void OnRotateFlip(RotateFlipType rft, bool bChangeSize = false);
	void UpdateBars(void);
	void CreateGif(void);
	void InvalidateDocument(void);

	// Our pice de rsistance: the very versatile QColorMatrix class!
	QColorMatrix m_ColorMatrix;

	// Our second pice de rsistance: the very nice QGaussFilter class!
	QGaussFilter m_Filter;

	Bitmap *m_pModified;
	Bitmap *m_pOriginal;
	Rect m_Rect;
	CSize m_szDoc;
	ULONG m_SaveQuality;
	REAL m_Gamma;
	bool m_bBorder;
	bool m_bViewModified;
	bool m_bImageResolution;
	COLORREF m_colBackGround;
	COLORREF m_colBorder;
	Bitmap *m_pGif;
	bool m_bGifOctree;
	bool m_bGifPreview;

	DECLARE_DYNCREATE(CTinterView)
	DECLARE_MESSAGE_MAP()
};

// COptionsDlg dialog

class COptionsDlg : public CDialog
{
public:
	COptionsDlg(CWnd* pParent = NULL);
	virtual ~COptionsDlg();
	int m_JPEGQuality;
	BOOL m_GifMode;
protected:
	enum { IDD = IDD_OPTIONS };
	virtual void DoDataExchange(CDataExchange* pDX);
	virtual BOOL OnInitDialog();
	afx_msg void OnHScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar);
	CSliderCtrl c_JPEGQuality;
	DECLARE_DYNAMIC(COptionsDlg)
	DECLARE_MESSAGE_MAP()
};

// CZoomLevelDlg dialog

class CZoomLevelDlg : public CDialog
{
public:
	CZoomLevelDlg(CWnd* pParent = NULL);
	virtual ~CZoomLevelDlg();
	enum { IDD = IDD_ZOOMLEVEL };
	int m_ZoomPercentage;
protected:
	virtual BOOL OnInitDialog();
	virtual void DoDataExchange(CDataExchange* pDX);
	CSpinButtonCtrl m_ZoomSpin;
	DECLARE_DYNAMIC(CZoomLevelDlg)
	DECLARE_MESSAGE_MAP()
};
